grammar JsonFormat ;

options {
	output=AST ;
}

tokens {
	NUMBER;
	MAP;   
	MAPLET;
	LIST;
	KEY;
	STRING;
	TRUE='true'; 
	FALSE='false'; 
	NULL='null';
}

@parser::header {
package com.springsource.json.parse.core.generated;
}

@parser::members {

	public void recoverFromMismatchedSet(IntStream input, RecognitionException e, BitSet follow) throws RecognitionException {
		throw e;
	}


	public void recoverFromMismatchedToken(IntStream input, RecognitionException e, int ttype,
										   BitSet follow)
		throws RecognitionException
	{
		throw e;
	}
	
}


@lexer::header {
package com.springsource.json.parse.core.generated;

}

@lexer::members {
}

@rulecatch {
	catch(RecognitionException e) {
		//reportError(e);
		throw e;
	}
}

// top level is either a map or a list
objects : (map | list) EOF ;

map   : ( '{' maplet ( ',' maplet )* '}' -> ^( MAP maplet+ ) ) | 
        ( '{' '}' -> ^( MAP ) );
	   
maplet : key ':' value -> ^( MAPLET key value );
      
key    : String  -> ^( KEY String ) ;

value  : literal | map | list;

literal: String | TRUE | FALSE | NULL | number;
		
list  : (  '[' ']' -> ^( LIST ) ) |
	    (  '[' value ( ',' value )* ']' -> ^( LIST value+ ) );


number:	SimpleNumber (Exponent)? -> ^(NUMBER SimpleNumber Exponent?);
SimpleNumber: ('-')? ('0' | NON_ZERO_PREFIXED_DIGITS) ( '.' ANY_DIGITS)?;
Exponent: ('e'|'E') ('+'|'-')? ANY_DIGITS;	
	
String 	: '"' ( EscapeSequence | ~('\u0000'..'\u001f' | '\\' | '\"' ) )*  '"' ;
fragment NON_ZERO_PREFIXED_DIGITS : DIGIT19 (DIGIT*); // digit 1-9 followed by any number of digits 0-9
fragment ANY_DIGITS : DIGIT (DIGIT*);  // any number of digits 0-9
fragment EscapeSequence
    	:   '\\' (UnicodeEscape |'b'|'t'|'n'|'f'|'r'|'\"'|'\''|'\\'|'/');
fragment UnicodeEscape : 'u' HexDigit HexDigit HexDigit HexDigit ;
fragment HexDigit : '0'..'9' | 'A'..'F' | 'a'..'f' ;
fragment DIGIT : '0' .. '9';
fragment DIGIT19 : '1' .. '9';
 
COMMENT      : '/*' .* '*/' { skip() ;} ;
LINE_COMMENT : '//' ~( '\n' | '\r' )* '\r'? '\n' { skip() ;} ;	
WHITESPACE   : ( ' ' | '\n' | '\r' | '\t' )+ { skip() ;} ;
