tree grammar JsonFormatTreeWalker ;
options {
	tokenVocab = JsonFormat ;
	ASTLabelType = CommonTree ;
}

@header {
	package com.springsource.json.parse.core.generated;
	
	import com.springsource.json.parse.core.Node;
	import com.springsource.json.parse.core.CollectionNode;
	import com.springsource.json.parse.core.ListNode;
	import com.springsource.json.parse.core.MapNode;
	import com.springsource.json.parse.core.ScalarNode;
	import com.springsource.json.parse.internal.StandardBooleanNode;
	import com.springsource.json.parse.internal.StandardNullNode;
	import com.springsource.json.parse.internal.StandardStringNode;
	import com.springsource.json.parse.internal.StandardNumberNode;
	import com.springsource.json.parse.internal.StandardListNode;
	import com.springsource.json.parse.internal.StandardMapNode;
}

@members {
	CollectionNode rootNode = null;

	// A stack is used to hold the current node to provide the parent when constructing a new node
	Stack<Node> current = new Stack<Node>();
	
	// Stacks are used to hold the last of various kinds of nodes but should never be more than one element deep.
	Stack<Node> valueStack = new Stack<Node>();
	Stack<MapNode> mapStack = new Stack<MapNode>();
	Stack<ListNode> listStack = new Stack<ListNode>();
	Stack<ScalarNode> literalStack = new Stack<ScalarNode>();
	
	public CollectionNode getRootNode() {
		return rootNode;
	}
	
	/**
	 * Return true if current node is being processed at the top of the tree.
	 */
	private boolean atRoot() {
	  return this.current.isEmpty();
	}
	
	protected void mismatch(IntStream input, int ttype, BitSet follow)
		throws RecognitionException
	{
	  
	  super.mismatch(input,ttype,follow);
    }
}

objects
	: (map | list)
	;

map
scope {
	StandardMapNode mapNode;
}
@init {
	$map::mapNode = new StandardMapNode((atRoot()?null:this.current.peek()));
	this.current.push($map::mapNode);
}
@after {
	this.current.pop();
	if (atRoot()) {
	  rootNode = $map::mapNode;
	} else {
	  mapStack.push($map::mapNode);
	}
}
	:	^( MAP (maplet)* )
	;

maplet
scope {
	String key;
	Node value;
}
@after {
	$map::mapNode.setNode($maplet::key, $maplet::value);
}
	:	^( MAPLET key value {$maplet::value = valueStack.pop();} )
	;


value 
	:	map {valueStack.push(mapStack.pop());} | 
	    list {valueStack.push(listStack.pop());} |
	    literal { valueStack.push(literalStack.pop());}
	;
	
	
literal	:	
 string | trueLiteral | falseLiteral | nullLiteral | number ;

list
scope {
	StandardListNode listNode;
}
@init {
	$list::listNode = new StandardListNode((atRoot()?null:this.current.peek()));
	this.current.push($list::listNode);
}
@after {
	this.current.pop();
	if (atRoot()) {
	    rootNode = $list::listNode;
	} else {
	    listStack.push($list::listNode);
	}
}
	: 	^( LIST ( value {$list::listNode.addNode(valueStack.pop());} )* )
	;
	
key	: 	^( KEY String {$maplet::key=$String.text;} );
			
string  		: ^( String {literalStack.push(new StandardStringNode(this.current.peek(), $String.text));} );
trueLiteral     : ^( TRUE {literalStack.push(new StandardBooleanNode(this.current.peek(), true));} );
falseLiteral    : ^( FALSE {literalStack.push(new StandardBooleanNode(this.current.peek(), false));} );
nullLiteral     : ^( NULL {literalStack.push(new StandardNullNode(this.current.peek()));} );

number  
scope {
  String base;
  String exponent;
}
@after {
  literalStack.push(new StandardNumberNode(this.current.peek(),$number::base+($number::exponent==null?"":$number::exponent)));
}
:
^( NUMBER (SimpleNumber { $number::base=$SimpleNumber.text;} )? (Exponent { $number::exponent=$Exponent.text;} )?  );

