#!/bin/bash

if [ -z "$JAVA_HOME" ]
then
    echo The JAVA_HOME environment variable is not defined
    exit 1
fi
	
SCRIPT="$0"

# SCRIPT may be an arbitrarily deep series of symlinks. Loop until we have the concrete path.
while [ -h "$SCRIPT" ] ; do 
  ls=`ls -ld "$SCRIPT"`
  # Drop everything prior to ->
  link=`expr "$ls" : '.*-> \(.*\)$'`
  if expr "$link" : '/.*' > /dev/null; then
    SCRIPT="$link"
  else
    SCRIPT=`dirname "$SCRIPT"`/"$link"
  fi
done
 
SERVER_HOME=`dirname "$SCRIPT"`/..
. $SERVER_HOME/bin/setupClasspath.sh
SERVER_HOME=`cd $SERVER_HOME; pwd`
 
#parse args for the script
cleanFlag=
debugFlag=
debugPort=8000
jmxPort=9875
CONFIG_DIR=$SERVER_HOME/config
jmxUsersPath=
keystorePath=
keystorePassword=changeit
suspend=n

shopt -s extglob

while (($# > 0))
	do
	case $1 in
	-debug) 
			debugFlag=1
			if [[ "$2" == +([0-9]) ]]
			then
				debugPort=$2
				shift;
			fi
			;;
	-clean)
			cleanFlag=1
			;;
	-configDir)
	        CONFIG_DIR=$2
	        shift;
	        ;;
	-jmxport)
			jmxPort=$2
			shift;
			;;
	-jmxusers)
			jmxUsersPath=$2
			shift;
			;;
	-keystore)
			keystorePath=$2
			shift;
			;;
	-keystorePassword)
			keystorePassword=$2
			shift;
			;;
	-suspend)
			suspend=y
	esac
	shift
done

if [[ "$CONFIG_DIR" != /* ]]
then
    CONFIG_DIR=$SERVER_HOME/$CONFIG_DIR
fi

if [ -z "$jmxUsersPath" ]
then
    jmxUsersPath=$CONFIG_DIR/control/jmx-users.properties
fi

if [ -z "$keystorePath" ]
then
    keystorePath=$CONFIG_DIR/control/keystore
fi

if [ "$debugFlag" ]
then
	DEBUG_OPTS=" \
		-Xdebug \
		-Xrunjdwp:transport=dt_socket,address=${debugPort},server=y,suspend=${suspend}"
fi

if [ "$cleanFlag" ]
then
	APP_OPTS=" \
		$APP_OPTS \
		-Dcom.springsource.server.clean=true"
fi

JMX_OPTS=" \
	$JMX_OPTS \
	-Dcom.sun.management.jmxremote.port=${jmxPort} \
	-Dcom.sun.management.jmxremote.authenticate \
	-Dcom.sun.management.jmxremote.password.file=${jmxUsersPath} \
	-Djavax.net.ssl.keyStore=${keystorePath} \
	-Djavax.net.ssl.keyStorePassword=${keystorePassword} \
	-Dcom.sun.management.jmxremote.ssl=true \
	-Dcom.sun.management.jmxremote.ssl.need.client.auth=false"


$JAVA_HOME/bin/java \
	$JAVA_OPTS \
	$DEBUG_OPTS \
	$APP_OPTS \
	$JMX_OPTS \
	-Dcom.springsource.server.home=$SERVER_HOME \
	-Dcom.springsource.server.configDir=$CONFIG_DIR \
	-Djava.io.tmpdir=$SERVER_HOME/work/tmp/ \
	-classpath $CLASSPATH com.springsource.server.kernel.bootstrap.Bootstrap
